(*
 * MainForm.pas - part PasLibVlcDemo pascal interface for libvlc 1.1.0 demo
 *
 * See copyright notice below.
 *
 * Last modified: 2010.10.01
 *
 * author: Robert Jdrzejczyk
 * e-mail: robert@prog.olsztyn.pl
 *    www: http://www.prog.olsztyn.pl/paslibvlc
 *
 * 2010.07.14 David Nottage, davidnottage@gmail.com
 *
 * Change PChar to PAnsiChar
 * This help usage in Delphi 2010
 *
 * 2010.09.02 Robert Jdrzejczyk
 *
 * Check working with libvlc 1.1.4
 *
 * 2010.09.11 Robert Jdrzejczyk
 *
 * Minor changes for better work under Windows 7
 * Execute Open Dialog
 *
 *
 * 2010.10.01 Robert Jdrzejczyk
 *
 * Split example code into 3 parts:
 *
 * PlayerInit
 * PlayerPlay
 * PlayerDone
 *
 *)

(* This is file is part of PasLibVlcPlayerDemo program
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Any non-GPL usage of this software or parts of this software is strictly
 * forbidden.
 *
 * The "appropriate copyright message" mentioned in section 2c of the GPLv2
 * must read: "Code from FAAD2 is copyright (c) Nero AG, www.nero.com"
 *
 * Commercial non-GPL licensing of this software is possible.
 * please contact robert@prog.olsztyn.pl
 *
 *)

(*
 * libvlc is part of project VideoLAN
 *
 * Copyright (c) 1996-2010 VideoLAN Team
 *
 * For more information about VideoLAN
 *
 * please visit http://www.videolan.org
 *
 *)

unit MainFormUnit;

interface

uses
  {$IFDEF UNIX}
  Unix,
  {$ENDIF}
  {$IFDEF MSWINDOWS}
  Windows,
  {$ENDIF}
  Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Menus,
  PasLibVlcUnit, PasLibVlcClassUnit, PasLibVlcPlayerUnit;

type
  TMainForm = class(TForm)
    OpenDialog1: TOpenDialog;
    MainMenu1: TMainMenu;
    MenuFile: TMenuItem;
    MenuFileOpen: TMenuItem;
    PopupMenu1: TPopupMenu;
    a1: TMenuItem;
    b1: TMenuItem;
    c1: TMenuItem;
    d1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure MenuFileOpenClick(Sender: TObject);
  private
    p_li: libvlc_instance_t_ptr;
    p_mi: libvlc_media_player_t_ptr;
    procedure PlayerInit(Handle: THandle);
    procedure PlayerPlay(fileName: WideString);
    procedure PlayerDone();
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.PlayerInit(Handle: THandle);
var
  args: array[0..1] of PAnsiChar;
begin
  libvlc_dynamic_dll_init();

  if (libvlc_dynamic_dll_error <> '') then
  begin
    MessageDlg(libvlc_dynamic_dll_error, mtError, [mbOK], 0);
    Close;
    exit;
  end;

  args[0] := PAnsiChar(libvlc_dynamic_dll_path);
  args[1] := NIL;

  p_li := libvlc_new(1, @args[0]);
  p_mi := libvlc_media_player_new(p_li);

  libvlc_video_set_key_input(p_mi, 1);
  libvlc_video_set_mouse_input(p_mi, 1);

  libvlc_media_player_set_hwnd(p_mi, Self.Handle);
end;

procedure TMainForm.PlayerPlay(fileName: WideString);
var
  p_md: libvlc_media_t_ptr;
begin
  // create new media from fileName
  p_md := libvlc_media_new_path(p_li, PAnsiChar(System.UTF8Encode(fileName)));

  // hide title
  libvlc_media_add_option(p_md, 'no-video-title-show');

  // assign media to player
  libvlc_media_player_set_media(p_mi, p_md);

  // play
  libvlc_media_player_play(p_mi);

  // release media
  if (p_md <> NIL) then
  begin
    libvlc_media_release(p_md);
    // p_md := NIL;
  end;
end;

procedure TMainForm.PlayerDone();
begin
  if (p_mi <> NIL) then
  begin
    libvlc_media_player_release(p_mi);
    p_mi := NIL;
  end;

  if (p_li <> NIL) then
  begin
    libvlc_release(p_li);
    p_li := NIL;
  end;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  PlayerDone();
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  PlayerInit(SELF.Handle);
end;

procedure TMainForm.MenuFileOpenClick(Sender: TObject);
begin
  if not OpenDialog1.Execute() then
  begin
    Close;
    exit;
  end;

  PlayerPlay(OpenDialog1.FileName);
end;

end.
