unit MainFormUnit;

interface

uses
  {$IFDEF UNIX}
  Unix,
  {$ENDIF}
  {$IFDEF MSWINDOWS}
  Windows,
  {$ENDIF}
  Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls,
  PasLibVlcUnit, PasLibVlcClassUnit, PasLibVlcPlayerUnit;

type
  TMainForm = class(TForm)
    Player: TPasLibVlcPlayer;
    PasLibVlcMediaList1: TPasLibVlcMediaList;
    PlayListBox: TListBox;
    BottomPanel: TPanel;
    PlayBtn: TButton;
    PrevBtn: TButton;
    NextBtn: TButton;
    StopBtn: TButton;
    ClearBtn: TButton;
    PlayModeNormalBtn: TButton;
    PlayModeLoopBtn: TButton;
    PlayModeRepeatBtn: TButton;
    RightPanel: TPanel;
    EventsMemo: TMemo;
    LeftPanel: TPanel;
    InfoPanel: TPanel;
    TimePanel: TPanel;
    TrackBar: TTrackBar;
    procedure FormCreate(Sender: TObject);
    procedure PlayListBoxClick(Sender: TObject);
    procedure PasLibVlcMediaList1NextItemSet(Sender: TObject; mrl: WideString;
      item: Pointer; index: Integer);
    procedure PasLibVlcMediaList1ItemAdded(Sender: TObject; mrl: WideString;
      item: Pointer; index: Integer);
    procedure PasLibVlcMediaList1ItemDeleted(Sender: TObject; mrl: WideString;
      item: Pointer; index: Integer);
    procedure PlayBtnClick(Sender: TObject);
    procedure PrevBtnClick(Sender: TObject);
    procedure NextBtnClick(Sender: TObject);
    procedure StopBtnClick(Sender: TObject);
    procedure ClearBtnClick(Sender: TObject);
    procedure PlayModeNormalBtnClick(Sender: TObject);
    procedure PlayModeLoopBtnClick(Sender: TObject);
    procedure PlayModeRepeatBtnClick(Sender: TObject);
    procedure PlayerMediaPlayerEndReached(Sender: TObject);
    procedure PlayerMediaPlayerPositionChanged(Sender: TObject;
      position: Single);
    procedure PlayerMediaPlayerLengthChanged(Sender: TObject; time: Int64);
    procedure PlayerMediaPlayerTimeChanged(Sender: TObject; time: Int64);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.PlayModeLoopBtnClick(Sender: TObject);
begin
  PasLibVlcMediaList1.SetPlayModeLoop();
end;

procedure TMainForm.PlayModeNormalBtnClick(Sender: TObject);
begin
  PasLibVlcMediaList1.SetPlayModeNormal();
end;

procedure TMainForm.PlayModeRepeatBtnClick(Sender: TObject);
begin
  PasLibVlcMediaList1.SetPlayModeRepeat();
end;

procedure TMainForm.ClearBtnClick(Sender: TObject);
begin
  PasLibVlcMediaList1.Clear();
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
//  not work in wersion 1.1.11
//  PasLibVlcMediaList1.Add('http://www.youtube.com/watch?v=4AnpQfd7PGI');
//  PasLibVlcMediaList1.Add('http://www.youtube.com/watch?v=Bh_sV058aLU');
//  PasLibVlcMediaList1.Add('http://www.youtube.com/watch?v=GJbo6ynoxEk');
//
//  PasLibVlcMediaList1.Add('../../_testFiles/Maximize.mp4');
//  PasLibVlcMediaList1.Add('../../_testFiles/test.ts');
//  PasLibVlcMediaList1.Add('../../_testFiles/rmf_fm.m3u');
//  PasLibVlcMediaList1.Add('http://www.radio-canada.ca/util/endirect/premiere.asx');
  PasLibVlcMediaList1.Add('../../_testFiles/Maximize.mp4');
  PasLibVlcMediaList1.Add('../../_testFiles/test.ts');
  PasLibVlcMediaList1.Add('../../_testFiles/rmf_fm_aacp.asx');
  PasLibVlcMediaList1.Add('http://www.radio-canada.ca/util/endirect/premiere.asx');
end;

procedure TMainForm.PlayListBoxClick(Sender: TObject);
var
  idx: Integer;
  item: TObject;
begin
  if (PlayListBox.ItemIndex < 0) then exit;
  if (PlayListBox.ItemIndex >= PlayListBox.Count) then exit;

  // check for item stil exists in play list
  item := PlayListBox.Items.Objects[PlayListBox.ItemIndex];
  idx := PasLibVlcMediaList1.IndexOfItem(item);

  // if exists then play it
  if (idx > -1) then
  begin
    PasLibVlcMediaList1.PlayItem(item);
  end;
end;

procedure TMainForm.NextBtnClick(Sender: TObject);
begin
  PasLibVlcMediaList1.Next();
end;

procedure TMainForm.PasLibVlcMediaList1ItemAdded(Sender: TObject;
  mrl: WideString; item: Pointer; index: Integer);
begin
  PlayListBox.Items.AddObject(mrl, item);
end;

procedure TMainForm.PasLibVlcMediaList1ItemDeleted(Sender: TObject;
  mrl: WideString; item: Pointer; index: Integer);
begin
  PlayListBox.Items.Delete(index);
end;

procedure TMainForm.PasLibVlcMediaList1NextItemSet(Sender: TObject;
  mrl: WideString; item: Pointer; index: Integer);
begin
  if (index < PlayListBox.Count) then
  begin
    PlayListBox.ItemIndex := index;
  end;

  EventsMemo.Lines.Add('Play ' + mrl);
end;

procedure TMainForm.PlayBtnClick(Sender: TObject);
begin
  PasLibVlcMediaList1.Play();
end;

procedure TMainForm.PlayerMediaPlayerEndReached(Sender: TObject);
begin
  TrackBar.Position := 0;
  TimePanel.Caption := '';
end;

procedure TMainForm.PlayerMediaPlayerLengthChanged(Sender: TObject;
  time: Int64);
begin
  TimePanel.Caption := Player.GetVideoPosStr('mm:ss') + ' / ' +
                       Player.GetVideoLenStr('mm:ss');
end;

procedure TMainForm.PlayerMediaPlayerPositionChanged(Sender: TObject;
  position: Single);
begin
  TrackBar.Position := Round(position);
end;

procedure TMainForm.PlayerMediaPlayerTimeChanged(Sender: TObject; time: Int64);
begin
  TimePanel.Caption := Player.GetVideoPosStr('mm:ss') + '/' +
                       Player.GetVideoLenStr('mm:ss');
end;

procedure TMainForm.PrevBtnClick(Sender: TObject);
begin
  PasLibVlcMediaList1.Prev();
end;

procedure TMainForm.StopBtnClick(Sender: TObject);
begin
  PasLibVlcMediaList1.Stop();
end;

end.


